#!/usr/bin/env python

import commands
import os
import glob
import logging
import time

logger = logging.getLogger()
logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.INFO )

pdb2MaePyText = '''import schrodinger.structure as ss
import sys
inFile = sys.argv[1]
outFile = sys.argv[2]
if not('.mae' in inFile):
    raise Exception('Invalid input file; must be .mae format')
struct = ss.StructureReader(inFile).next()
print struct
struct.write(outFile)
'''

def dock(ligand_pdbqt,  protein_pdbqt, grid_center,):
    center_list = grid_center.split(',')
    vina_command = 'vina --receptor %s  --ligand %s --center_x %s --center_y %s --center_z %s --size_x 15 --size_y 15 --size_z 15 --seed 999 >& vina_output' %(protein_pdbqt, ligand_pdbqt, center_list[0], center_list[1], center_list[2])
    out_dock_file = ligand_pdbqt.replace('.pdbqt','_out.pdbqt')

    commands.getoutput(vina_command)
    if not(os.path.isfile(out_dock_file)):
        logging.info("Docking failed for protein %s and ligand %s" %(protein_pdbqt, ligand_pdbqt))
        return False
    return out_dock_file

def main_vina (stage_3_result, stage_4_working, update= True):
    os.chdir(stage_4_working)
    current_dir = os.getcwd()
    dockable_paths = []
    for all_pdb_path in os.walk(stage_3_result):
        if all_pdb_path[0] != stage_3_result:
            dockable_paths.append(all_pdb_path[0])

    for dockable_path in dockable_paths:
        commands.getoutput("cp -r %s %s" %(dockable_path, stage_4_working))
        target_name = os.path.basename(dockable_path)
        os.chdir(target_name)
        current_dir_layer_2 = os.getcwd()
        ##################
        #Do the docking here
        
        #1, get the center
        try:
            grid_center = open("center.txt","r").readlines()[0]
            logging.info("=============Start working on this case:%s=========="%target_name)
        except:
            logging.info("Fatal error: Unable to find the center file for this case %s"%target_name)
            os.chdir(current_dir)
            continue
        
        # Get the candidate protein names in this directory
        candidate_proteins = glob.glob('./*-????_????_prepared.mol2')
        
        logging.info('Found candidates %r' %(candidate_proteins))
        # Get the ligand names in this directory
        ligand_mol2s = glob.glob('lig_*_prepared.mol2')
        ligand_mol2s = [i for i in ligand_mol2s if not "unprep" in i]
        if len(ligand_mol2s) == 0:
            logging.info('No ligand files found for target %s. Skipping.' %(dockable_path))
            os.chdir(current_dir)
            continue
        if len(ligand_mol2s) > 1:
            logging.info('Multiple ligand files found for target %s (found ligands %r). The workflow currently should only be sending one ligand. Skipping. ' %(dockable_path, ligand_mol2s))
            os.chdir(current_dir)
            continue

        
        
        for candidate_protein in candidate_proteins:
            candidate_prefix = candidate_protein.replace('_prepared.mol2','')
            #if os.path.isfile(candidate_protein):
            logging.info( "Working on this receptor: %s"%candidate_protein )
            if not os.path.isdir(candidate_prefix):
                os.mkdir(candidate_prefix)
            os.chdir(candidate_prefix)
            ##################
            #do docking inside
            commands.getoutput("cp ../%s ."%candidate_protein)
            
            ## Technical prep: Prepare the protein
            #receptorMol2File = '.'.join(possible_protein.split('.')[:-1]) + '.mol2'
            commands.getoutput('. /usr/local/mgltools/bin/mglenv.sh; pythonsh $MGL_ROOT/MGLToolsPckgs/AutoDockTools/Utilities24/prepare_receptor4.py -r %s' %(candidate_protein))
            receptorPdbqtFile = candidate_protein.replace('.mol2','.pdbqt')


            for ligand_mol2 in ligand_mol2s:
                ## Technical prep: Prepare the ligand
                ligand_name = ligand_mol2.replace('lig_','').replace('_prepped.mol2','')
                ligandPdbqtFile = ligand_mol2.replace('.mol2','.pdbqt')
                commands.getoutput("cp ../%s ." %(ligand_mol2))
                commands.getoutput('. /usr/local/mgltools/bin/mglenv.sh; pythonsh $MGL_ROOT/MGLToolsPckgs/AutoDockTools/Utilities24/prepare_ligand4.py -l %s' %(ligand_mol2))
    
                ## Do the docking
                logging.info("Trying to dock...")
                out_dock_file = dock(ligandPdbqtFile, receptorPdbqtFile, grid_center)
                if out_dock_file == False:
                    logging.info("Docking failed - Skipping")
                    continue
                logging.info("Finished docking, beginning post-docking file conversion")
                
                ## Convert the receptor to pdb
                intermediates_prefix = '%s_postdocking' %(candidate_prefix)
                output_prefix = '%s_docked' %(candidate_prefix)
                #receptorPdbqt = out_dock_file.replace('ligand_out',candidate_name)
                #receptorPdbqt = candidate_prefix+'.pdbqt'
                ## This receptor pdb will be one of our final outputs
                outputReceptorPdb = "%s.pdb" %(output_prefix)
                commands.getoutput('. /usr/local/mgltools/bin/mglenv.sh; python $MGL_ROOT/MGLToolsPckgs/AutoDockTools/Utilities24/pdbqt_to_pdb.py -f %s -o %s' %(receptorPdbqtFile, outputReceptorPdb))
                
                ## Then make the ligand mol
                ## pdbqt_to_pdb.py can't split up the multiple poses in vina's output files, so we do that by hand
                with open(out_dock_file) as fo:
                    fileData = fo.read()
                fileDataSp = fileData.split('ENDMDL')
                
                ## Write out each pose to its own pdb and mol file, then merge with the receptor to make the complex files.
                for index, poseText in enumerate(fileDataSp[:-1]):
                    this_pose_pdbqt = intermediates_prefix+'_ligand'+str(index+1)+'.pdbqt'
                    this_pose_pdb = intermediates_prefix+'_ligand'+str(index+1)+'.pdb'
                    this_pose_mol = intermediates_prefix+'_ligand'+str(index+1)+'.mol'
                    with open(this_pose_pdbqt,'wb') as wf:
                        wf.write(poseText+'ENDMDL')
                    commands.getoutput('. /usr/local/mgltools/bin/mglenv.sh; python $MGL_ROOT/MGLToolsPckgs/AutoDockTools/Utilities24/pdbqt_to_pdb.py -f %s -o %s' %(this_pose_pdbqt, this_pose_pdb))
                    commands.getoutput('babel -ipdb %s -omol %s' %(this_pose_pdb, this_pose_mol))
                
                ## Here convert our top-ranked pose to the final submission for this docking
                ## Right now we're ignoring everything other than the top pose
                top_intermediate_mol = intermediates_prefix+'_ligand1.mol'
                final_ligand_mol = output_prefix+'.mol'
                commands.getoutput('cp %s %s' %( top_intermediate_mol, final_ligand_mol))
                commands.getoutput('cp %s ../' %(final_ligand_mol))
                commands.getoutput('cp %s ../' %(outputReceptorPdb))
                    
                ##################
            os.chdir(current_dir_layer_2)
        os.chdir(current_dir)
        ##################

if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-s", "--structuredir", metavar="PATH", help = "PATH where we can find the proteinligprep output")
    parser.add_argument("-o", "--outdir", metavar = "PATH", help = "PATH where we will put the docking output")
    parser.add_argument("-u", "--update", action = "store_true",  help = "Update the docking result", default = False) 
    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.INFO )
    opt = parser.parse_args()
    proteinligprep_dir = opt.structuredir
    dock_result_dir = opt.outdir
    update = opt.update
    #running under this dir
    running_dir = os.getcwd()
    main_vina(proteinligprep_dir, dock_result_dir, update = update)
    #move the final log file to the result dir
    log_file_path = os.path.join(running_dir, 'final.log')
    commands.getoutput("mv %s %s"%(log_file_path, dock_result_dir))



__author__ = 'robswift'


import os
import logging
from d3r.filter.filter import QueryFilter
from d3r.filter.filter import HitFilter

logger = logging.getLogger(__name__)


class InputAnalysis(object):
    """

    """
    def __init__(self, targets):
        """

        :param targets:
        :return:
        """
        self.targets = targets
        self.number_of_structures = None            # total number of structures in targets
        self.number_of_complexes = None             # total number of structures with at least one bound ligand
        self.one_dock = None                        # total number of structures with only one dockable ligand
        self.at_least_one_dock = None               # total number of structures with at least one dockable ligand
        self.number_of_monomers = None              # total number of structures that are monomers
        self.number_of_monomers_one_dock = None     # total number of structures that are monomers with one dockable ->
                                                    # -> ligand.
        self.number_of_multimers = None             # total number of structures with more than one unique chain.
        self.number_of_multimers_one_dock = None    # total number of structures that are multimers with one dockable ->
                                                    # -> ligand.

    def set_number_of_structures(self):
        if self.number_of_structures:
            pass
        else:
            self.number_of_structures = len(self.targets)

    def set_number_of_complexes(self):
        if self.number_of_complexes:
            pass
        else:
            self.number_of_complexes = 0
            for target in self.targets:
                if int(target.ligand_count) > 0:
                    self.number_of_complexes += 1

    def set_one_dock(self):
        if self.one_dock:
            pass
        else:
            self.one_dock = 0
            for target in self.targets:
                if int(target.dock_count) == 1:
                    self.one_dock += 1

    def set_at_least_one_dock(self):
        if self.at_least_one_dock:
            pass
        else:
            self.at_least_one_dock = 0
            for target in self.targets:
                if int(target.dock_count) > 0:
                    self.at_least_one_dock += 1

    def set_number_of_monomers(self):
        if self.number_of_monomers:
            pass
        else:
            self.number_of_monomers = 0
            for target in self.targets:
                if int(target.sequence_count) == 1:
                    self.number_of_monomers += 1

    def set_number_of_monomers_one_dock(self):
        if self.number_of_monomers_one_dock:
            pass
        else:
            self.number_of_monomers_one_dock = 0
            for target in self.targets:
                if int(target.sequence_count) == 1 and int(target.dock_count) == 1:
                    self.number_of_monomers_one_dock += 1

    def set_number_of_multimers(self):
        if self.number_of_multimers:
            pass
        else:
            self.number_of_multimers = 0
            for target in self.targets:
                if int(target.sequence_count) > 1:
                    self.number_of_multimers += 1

    def set_number_of_multimers_one_dock(self):
        if self.number_of_multimers_one_dock:
            pass
        else:
            self.number_of_multimers_one_dock = 0
            for target in self.targets:
                if int(target.sequence_count) > 1 and int(target.dock_count) == 1:
                    self.number_of_multimers_one_dock += 1

    def print_to_standard_out(self):
        self.set_number_of_structures()
        self.set_number_of_complexes()
        self.set_at_least_one_dock()
        self.set_one_dock()
        self.set_number_of_monomers()
        self.set_number_of_monomers_one_dock()
        self.set_number_of_multimers()
        self.set_number_of_multimers_one_dock()
        print "The total number of structures is: %s" % self.number_of_structures
        print "The total number of complexes is: %s" % self.number_of_complexes
        print "The total number of complexes with at least one dockable ligand is: %s" % self.at_least_one_dock
        print "The total number of complexes with only one dockable ligand is: %s" % self.one_dock
        print "The total number of monomers is: %s" % self.number_of_monomers
        print "The total number of monomers with one dockable ligand is: %s" % self.number_of_monomers_one_dock
        print "The total number of multimers is: %s" % self.number_of_multimers
        print "The total number of multimers with one dockable ligand is: %s" % self.number_of_multimers_one_dock

    def print_to_file(self, out_dir):
        self.set_number_of_structures()
        self.set_number_of_complexes()
        self.set_at_least_one_dock()
        self.set_one_dock()
        self.set_number_of_monomers()
        self.set_number_of_monomers_one_dock()
        self.set_number_of_multimers()
        self.set_number_of_multimers_one_dock()
        logger.debug('Writing summary.txt file to ' + out_dir)
        handle = open(os.path.join(out_dir,'summary.txt'), 'w')
        out = ['INPUT SUMMARY']
        out.append('  entries:{no:>32}'.format(no = self.number_of_structures))
        out.append('  complexes:{no:>30}'.format(no = self.number_of_complexes))
        out.append('  dockable complexes:{no:>21}'.format(no = self.one_dock))
        out.append('  monomers:{no:>31}'.format(no = self.number_of_monomers))
        out.append('  dockable monomers:{no:>22}'.format(no = self.number_of_monomers_one_dock))
        out.append('  multimers:{no:>30}'.format(no = self.number_of_multimers))
        out.append('  dockable multimers:{no:>21}'.format(no = self.number_of_multimers_one_dock))
        out.append('')
        for l in out:
            logger.debug(l)
            handle.write("%s\n" % l)
        handle.close()

class OutputAnalysis(object):
    """

    """
    targets = {}                    # {'PDB ID' : (sequence_count, no. of blast hits, no. of candidates, no. elected,
                                    #              pdb_ids)

    def __init__(self, query = None):
        if query is not None:
            self.query = query

    def count_hits(self):
        return len(self.query.hits)

    def candidate_count(self):
        return len([hit for hit in self.query.hits if not hit.triage])

    def elected_count(self):
        return len([hit for hit in self.query.hits if hit.retain])

    def pdb_ids(self):
        return ','.join([hit.pdb_id for hit in self.query.hits if hit.retain])

    def set_target_dict(self):
        pdb_id = self.query.pdb_id
        seq_count = self.query.sequence_count
        hit_count = self.count_hits()
        candidate_count = self.candidate_count()
        elected_count = self.elected_count()
        pdb_ids = self.pdb_ids()
        OutputAnalysis.targets[pdb_id] = (seq_count, hit_count, candidate_count, elected_count, pdb_ids)

    def print_filter_criteria(self, out_dir):
        handle = open(os.path.join(out_dir, 'summary.txt'), 'a')
        out = ['FILTERING CRITERIA']
        out.append('  No. of query sequences           <=    {no}'.format(no = QueryFilter.sequence_threshold))
        out.append('  No. of dockable ligands           =    {no}'.format(no = QueryFilter.dockable_ligand_threshold))
        out.append('  Percent identity                 >=    {no:.2}'.format(no = HitFilter.identity_threshold))
        out.append('  Percent Coverage                 >=    {no:.2}'.format(no = HitFilter.coverage_threshold))
        out.append('  No. of hit sequences             <=    {no}'.format(no = HitFilter.sequence_threshold))
        out.append('  Structure determination method:        {method}'.format(method = HitFilter.method))
        out.append('')
        for l in out:
            logger.debug(l)
            handle.write("%s\n" %l)
        handle.close()

    def print_to_file(self, out_dir):
        handle = open(os.path.join(out_dir, 'summary.txt'), 'a')
        out = ['OUTPUT SUMMARY']
        out.append('  Targets found:{no:>26}'.format(no=len(OutputAnalysis.targets.keys())))
        for query in OutputAnalysis.targets.keys():
            sc, hc, cc, ec, pdb_ids = OutputAnalysis.targets[query]
            out.append('  Target: {id}|Sequences: {sc}|Hits: {hc}|'
                       'Candidates: {cc}|Elected:{ec}|PDBids: {pdbs}'.format(id=query, sc=sc, hc=hc, cc=cc, ec=ec,
                                                                             pdbs=pdb_ids))
        for l in out:
            logger.debug(l)
            handle.write("%s\n" %l)
        handle.close()


__author__ = 'j-wags'

import logging
import glob
import os

class ChallengeData(object):
    """
    A class for interacting with challenge data packages
    """
    
    def __init__(self, get_challenge_data_dir):
        self.get_challenge_data_dir = get_challenge_data_dir
        self.abs_get_challenge_data_dir = os.path.abspath(get_challenge_data_dir)
        
        self.weekly_challenge_packages = glob.glob('%s/celpp_week*/' %(self.abs_get_challenge_data_dir))
        self.weekly_challenge_folder_names = []
        print 'self.weekly_challenge_packages',self.weekly_challenge_packages

        self.week_challenge_dict = {}
        for wcp in self.weekly_challenge_packages:
            print 'wcp', wcp
            wcp_folder_name = wcp.strip('/').split('/')[-1]
            self.weekly_challenge_folder_names.append(wcp_folder_name)
            print 'wcp_folder_name', wcp_folder_name
            targets = glob.glob('%s/????/' %(wcp))
            print 'targets', targets
            self.week_challenge_dict[wcp_folder_name] = targets
            
        

    def get_week_names(self):
        """
        Gets the bottom-level folder names in this challenge pacakge ("celpp_weekXX_XXXX"), with slashes removed
        :return: list of strings
        """
        return self.weekly_challenge_folder_names
    
    def get_weekly_challenge_packages(self):
        """
        A function to return the all challenge data packages inside of this directory. Expects subdirectory contining a folder named, for example, celpp_week25_2016. Returns a list of absolute folder paths
        :return: A list of strings, each an absolute path to a challenge data package
        """
        
        return self.weekly_challenge_packages
                         


    def is_valid_for_celpp(self):
        """
        Determines whether the challenge data package contains exactly one week with at least one target
        :return: boolean
        """
        
        # Ensure that exactly one week is in here
        if len(self.week_challenge_dict.keys()) != 1:
            logging.info("Challenge package contains more than 1 week and is therefore not suitable for CELPP (contains %r)" %(self.week_challenge_dict.keys()))
            return False

        # Ensure that the week has at least one target
        if len(self.week_challenge_dict[self.week_challenge_dict.keys()[0]]) == 0:
            logging.info("Week in challenge package does not contain any targets (week folder %s)" %(self.week_challenge_dict[self.week_challenge_dict.keys()[0]]))
            return False

        return True

    def get_targets(self):
        """
        Returns the {week:[targets]} dictionary for this challenge package. If is_valid_for_celpp is True, this dictionary will have one key, with the corresponding value being a list of strings of absolute directories of targets.
        :return: dict
        """
        
        return self.week_challenge_dict
        



#!/usr/bin/env python

import logging
import shutil
import os
import glob
import tarfile
from d3r.celpp.filetransfer import WebDavFileTransfer
from d3r.utilities.challenge_data import ChallengeData

__author__ = 'j5wagner'
    

def find_uploadable_results(target_dir):
    # Search through a dock_dir/target_name dir to find files that
    # fit the upload format requirements.
    
    valid_results = []
    abs_targ_dir = os.path.abspath(target_dir)
    potential_pdbs = glob.glob('%s/*-????_????_docked.pdb'%(abs_targ_dir))
    
    for potential_pdb in potential_pdbs:

        ## Do validity checks
        if os.path.getsize(potential_pdb) == 0:
            logging.info('Size of %s is 0. Skipping candidate.'%(potential_pdb))
            continue
        potential_mol = potential_pdb.replace('_docked.pdb','_docked.mol')
        if not(os.path.exists(potential_mol)):
            logging.info('For candidate %s, I was unable to find docked ligand %s Skipping candidate.' %(potential_pdb, potential_mol))
            continue
        if os.path.getsize(potential_mol) == 0:
            logging.info('Size of %s is 0. Skipping candidate.'%(potential_mol))
            continue
        
        # If the target is valid, add it to the packing list
        valid_results.append((potential_pdb, potential_mol))
    
    return valid_results

def make_result_dictionary(dock_dir):
    ## Check every dir that's names like a 4-character target id
    pot_targ_dirs = glob.glob(os.path.join(dock_dir,'????/'))
    result_dic = {}
    for pot_targ_dir in pot_targ_dirs:
        pot_targ_id = os.path.basename(pot_targ_dir.strip('/'))
        valid_results = find_uploadable_results(pot_targ_dir)
        if valid_results == []:
            logging.info('No valid results found for target %s. Skipping target.' %(pot_targ_id))
            continue
        logging.info('Found valid results %r for target %s.' %(valid_results, pot_targ_id))
        result_dic[pot_targ_id] = valid_results
    return result_dic


def main_pack_dock_results(challenge_dir, dock_dir, pack_dir, ftp_config):
    abs_orig_dir = os.getcwd()
    abs_challenge_dir = os.path.abspath(challenge_dir)
    abs_pack_dir = os.path.abspath(pack_dir)

    ## Parse FTP config if given
    if ftp_config is None:
        abs_ftp_config = None
        contestant_id = 'XXXXX'
    else:
        abs_ftp_config = os.path.abspath(ftp_config)
        from d3r.celpp import filetransfer
        f_f_t_obj = WebDavFileTransfer(abs_ftp_config)
        contestant_id = f_f_t_obj.get_contestant_id()

    ## Get this week's name
    chal_data_obj = ChallengeData(abs_challenge_dir)
    week_name = chal_data_obj.get_week_names()[0]

    ## Find all possible uploadable target dirs
    result_dic = make_result_dictionary(dock_dir)
    
    submission_base_name = week_name + '_dockedresults_' + contestant_id 

    
    ## Copy them into this directory
    os.chdir(abs_pack_dir)
    os.mkdir(submission_base_name)
    os.chdir(submission_base_name)
    for targ_id in result_dic:
        os.mkdir(targ_id)
        for docked_pdb, docked_mol in result_dic[targ_id]:
            d_f_basename = os.path.basename(docked_pdb)
            destination = os.path.join(abs_pack_dir,
                                       submission_base_name,
                                       targ_id,
                                       d_f_basename)
            shutil.copyfile(docked_pdb, destination)
            
            d_f_basename = os.path.basename(docked_mol)
            destination = os.path.join(abs_pack_dir,
                                       submission_base_name,
                                       targ_id,
                                       d_f_basename)
            shutil.copyfile(docked_mol, destination)
            
            ### TEMPORARY HACK UNTIL EVALUATE GETS FIXED ###
            #hack_folder_name = destination.replace('_docked.mol','')
            #os.mkdir(hack_folder_name)
            #destination = os.path.join(hack_folder_name, d_f_basename)
            #shutil.copyfile(docked_mol, destination)
            #destination = destination.replace('.mol','.pdb')
            #shutil.copyfile(docked_pdb, destination)

    
    
    ## Tar up the pack directory. To keep this tarball from containing
    ## absolute paths, we enter the pack directory, go one directory
    ## up, and pack the tarball from there (thereby giving the content
    ## reasonable relative paths)
    os.chdir(abs_pack_dir)
    #os.chdir('..')
    
    #abs_tar_name = abs_pack_dir.rstrip('/') + '.tar.gz'
    tar_base_name = submission_base_name + '.tar.gz'
    abs_tar_name = os.path.join(abs_pack_dir, tar_base_name)
    
    logging.info('Creating tarfile %s in directory %s' %(abs_tar_name, os.getcwd()))
    tarfile_obj = tarfile.open(abs_tar_name, 'w:gz')
    logging.info('Writing to tarfile')
    #tarfile_obj.add(os.path.basename(abs_pack_dir.rstrip('/')))
    tarfile_obj.add(submission_base_name)
    tarfile_obj.close()
    logging.info('Tarfile closed')


    ## Use ftp config to upload tarball
    if ftp_config is None:
        logging.info('No ftp_config file given. Skipping upload')
        return

    #tar_base_name = os.path.basename(abs_tar_name)
    f_f_t_obj.connect()
    submission_dir = os.path.join('/dav/',
                                  #f_f_t_obj.get_remove_submission_dir, 
                                  contestant_id)
    f_f_t_obj.upload_file_direct(abs_tar_name,
                                 submission_dir,
                                 #f_f_t_obj.get_remote_submission_dir(),
                                 #'/dav/celppweekly/usersubmissions/12345/',
                                 tar_base_name)
    logging.info(f_f_t_obj.get_upload_summary())
    f_f_t_obj.disconnect()


        

if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-c", "--challengedata", metavar="PATH", help = "PATH to the unpacked challenge data package")
    parser.add_argument("-d", "--dockdir", metavar = "PATH", help = "Dir where docking was performed")
    parser.add_argument("-p", "--packdir", metavar="PATH", help = "Dir where the packing and uploading will be performed. Note that, for competition entries, this expects this dir name to already have the proper formatting for this contestant's entry, and will be used as the base of the tar.gz file.")
    parser.add_argument("-f", "--ftpconfig", metavar="PATH", help = "File containing user ftp config information (see included example ftp config for specifics)")
    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level = logging.INFO )
    args = parser.parse_args()
    challenge_dir = args.challengedata
    dock_dir = args.dockdir
    pack_dir = args.packdir
    ftp_config = args.ftpconfig
    
    abs_running_dir = os.getcwd()
    log_file_path = os.path.join(abs_running_dir, 'final.log')
    log_file_dest = os.path.join(os.path.abspath(pack_dir), 'final.log')

    main_pack_dock_results(challenge_dir, dock_dir, pack_dir, ftp_config)

    #move the final log file to the result dir
    shutil.move(log_file_path, log_file_dest)



__author__ = 'churas'

import unittest


"""
test_run
--------------------------------

Tests for `run` module.
"""

from d3r.utilities import run
from d3r import blastnfilter


class TestBlastNFilterTask(unittest.TestCase):
    def setUp(self):
        pass

    def test_split_input(self):
        theargs = ['--outdir', '/f/myoutdir',
                   '--nonpolymertsv', '/f/mynonpolymertsv',
                   '--sequencetsv', '/f/mysequencetsv',
                   '--crystalpH', '/f/mycrystalph',
                   '--pdbblastdb', '/f/mypdbblastdb',
                   '--pdbdb', '/f/mypdbdb',
                   '--compinchi', '/f/mycompinchi']
        result = blastnfilter._parse_arguments('hi', theargs)
        non_polymer, polymer, ph, out_dir, blast_dir, pdb_db, pdb_path,\
            fasta, compinchi = run.split_input(result)
        self.assertEqual(non_polymer, '/f/mynonpolymertsv')
        self.assertEqual(polymer, '/f/mysequencetsv')
        self.assertEqual(ph, '/f/mycrystalph')
        self.assertEqual(out_dir, '/f/myoutdir')
        self.assertEqual(blast_dir, '/f/mypdbblastdb')
        self.assertEqual(pdb_db, '/f/mypdbblastdb/pdb_db')
        self.assertEqual(pdb_path, '/f/mypdbdb')
        self.assertEqual(fasta, '/f/mypdbblastdb/pdb_seqres.txt')
        self.assertEqual(compinchi, '/f/mycompinchi')

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import os
import time
from ftpretty import ftpretty
import easywebdav

logger = logging.getLogger(__name__)


class InvalidFtpConfigException(Exception):
    """Thrown when an invalid ftp configuration is passed to FtpFileUploader
    """
    pass


class FileTransfer(object):
    """FileTransfer
    """
    HOST = 'host'
    USER = 'user'
    PASS = 'pass'
    PATH = 'path'
    CONTESTANTID = 'contestantid'
    CHALLENGEPATH = 'challengepath'
    SUBMISSIONPATH = 'submissionpath'

    def __init__(self, config):
        """Constructor

        """
        self._connect_timeout = 60
        self._ftp = None
        self._files_transferred = 0
        self._bytes_transferred = 0
        self._duration = 0
        self._alt_ftp_con = None
        self._ftp_host = None
        self._ftp_user = None
        self._ftp_pass = None
        self._remote_dir = None
        self._error_msg = None
        self._contestant_id = None
        self._remote_challenge_dir = None
        self._remote_submission_dir = None

        if config is not None:
            self._parse_config(config)

    def _parse_config(self, config):
        """Parses ftp config file for user credentials

           The parsed values are stored internally in this
           class.

           Expected format of `ftp_config` file

           host <HOST>
           user <USERNAME>
           pass <PASSWORD>
           path <BASE PATH ON REMOTE SERVER ie /foo>
           contestantid <CELPP CONTESTANT ID>
           challengepath <BASE PATH ON REMOTE SERVER ie /challenge>
           submissionpath <BASE PATH ON REMOTE SERVER ie /submission>

           Example:

           host ftp.box.com
           user bob@bob.com
           pass 12345
           path /upload
           contestantid 24680
           challengepath /challenge
           submissionpath /usersubmissions

           The above format matches the standard used
           by NCFTP with the exception of `path` and `contestantid`
           which are custom to this class

           :param ftp_config: Path to ftp config file
           :raises IOError: If there was an error opening the file
        """
        if config is None:
            raise InvalidFtpConfigException('No ftp file specified')

        f = None
        try:
            f = open(config, 'r')
            for line in f:
                split_line = line.split(' ')
                if not len(split_line) == 2:
                    continue
                if split_line[0] == FileTransfer.HOST:
                    self.set_host(split_line[1].rstrip())
                elif split_line[0] == FileTransfer.USER:
                    self.set_user(split_line[1].rstrip())
                elif split_line[0] == FileTransfer.PASS:
                    self.set_password(split_line[1].rstrip())
                elif split_line[0] == FileTransfer.PATH:
                    self.set_remote_dir(split_line[1].rstrip())
                elif split_line[0] == FileTransfer.CONTESTANTID:
                    self.set_contestant_id(split_line[1].rstrip())
                elif split_line[0] == FileTransfer.CHALLENGEPATH:
                    self.set_remote_challenge_dir(split_line[1].rstrip())
                elif split_line[0] == FileTransfer.SUBMISSIONPATH:
                    self.set_remote_submission_dir(split_line[1].rstrip())

        finally:
            if f is not None:
                f.close()

    def set_connection(self, ftp_con):
        """Lets caller use library other then ftpretty for ftp connections

           If set it is assumed `ftp_con` is already connected to
           ftp server and it supports the API defined by ftpretty:
           ftp_con.close()
           ftp_con.put('/pathtofile','/destinationpath')
        """
        self._alt_ftp_con = ftp_con

    def get_contestant_id(self):
        """Gets the contestant id
        """
        return self._contestant_id

    def set_contestant_id(self, contestant_id):
        """Sets the contestant id
        :param contestant_id: Id of contestant any value passed in other
        then None will be converted to a string
        """
        if contestant_id is None:
            self._contestant_id = None
            return

        self._contestant_id = str(contestant_id)

    def set_remote_challenge_dir(self, challenge_dir):
        """Sets the remote challenge directory for ftp upload
        """
        self._remote_challenge_dir = challenge_dir

    def get_remote_challenge_dir(self):
        """Gets the remote challenge directory for ftp upload
        """
        return self._remote_challenge_dir

    def set_remote_submission_dir(self, submission_dir):
        """Sets the remote submission directory for ftp download
        """
        self._remote_submission_dir = submission_dir

    def get_remote_submission_dir(self):
        """Gets the remote submission directory for ftp download
        """
        return self._remote_submission_dir

    def set_remote_dir(self, remote_dir):
        """Sets the remote directory where ftp files will be uploaded to
        """
        self._remote_dir = remote_dir

    def get_remote_dir(self):
        """Gets remote directory prefix path
           :returns: Remote directory prefix path, if value is None empty str
                     is returned
        """
        if self._remote_dir is None:
            return ''
        return self._remote_dir

    def set_host(self, ftp_host):
        """Sets ftp host to connect to
        :param: host should be hostname ie ftp.box.com
        """
        self._ftp_host = ftp_host

    def get_host(self):
        """Gets ftp host
        """
        return self._ftp_host

    def set_user(self, ftp_user):
        self._ftp_user = ftp_user

    def get_user(self):
        return self._ftp_user

    def set_password(self, ftp_pass):
        self._ftp_pass = ftp_pass

    def get_password(self):
        return self._ftp_pass

    def set_connect_timeout(self, timeout):
        self._connect_timeout = timeout

    def get_connect_timeout(self):
        return self._connect_timeout

    def get_error_msg(self):
        return self._error_msg

    def connect(self):
        """Dummy implementation
        """
        return True

    def disconnect(self):
        """Dummy implementation
        """
        pass

    def delete_file(self, remote_file):
        """Dummy implementation
        """
        self._error_msg = 'delete_file not implemented'
        return False

    def download_file(self, remote_file, local_file):
        """Dummy implementation
        """
        self._error_msg = 'download_file not implemented'
        return False

    def list_dirs(self, remote_dir):
        """Dummy
        """
        self._error_msg = 'list_dirs not implemented'
        return None

    def list_files(self, remote_dir):
        """Dummy
        """
        self._error_msg = 'list_files not implemented'
        return None

    def upload_file_direct(self, file, remote_dir, remote_file_name):
        """Dummy
        """
        self._error_msg = 'upload_file_direct not implemented'
        return False

    def upload_files(self, list_of_files):
        """Dummy
        """
        self._error_msg = 'upload_files not implemented'
        return False

    def get_upload_summary(self):
        """Gets summary of previous `upload_files` invocation

            :returns: Human readable string summary of format
            # files (# bytes) files uploaded in # seconds to
            host HOST:REMOTE_DIR
        """
        summary = ''
        if self._error_msg is not None:
            summary = self._error_msg + '\n'

        try:
            logger.debug('host: ' + self.get_host())
            host = self.get_host()
        except TypeError:
            logger.exception('Host not set')
            host = 'Unset'

        remote_dir = self.get_remote_dir()
        logger.debug('remote dir: ' + remote_dir)

        summary += (str(self._files_transferred) + ' (' +
                    str(self._bytes_transferred) +
                    ' bytes) files uploaded in ' +
                    str(self._duration) + ' seconds to host ' +
                    host + ':' + remote_dir)
        logger.debug('upload summary: ' + summary)
        return summary


class FtpFileTransfer(FileTransfer):
    """Implements FileUploader class by enabling upload of files to ftp

    """
    def __init__(self, config):
        """Constructor
        """
        super(FtpFileTransfer, self).__init__(config)

    def connect(self):
        if self._alt_ftp_con is None:
            try:

                logger.debug('Connecting to ' +
                             str(self.get_host()) + ' with user ' +
                             str(self.get_user()))
                self._ftp = ftpretty(self.get_host(),
                                     self.get_user(),
                                     self.get_password(),
                                     timeout=self.get_connect_timeout())
                return True
            except:
                logger.exception('Unable to connect to ftp host')
                self._error_msg = 'Unable to connect to ftp host'
                return False
        self._ftp = self._alt_ftp_con
        return True

    def disconnect(self):
        if self._ftp is None:
            logger.debug('ftp connection is None, just returning')
            return

        if self._alt_ftp_con is not None:
            logger.debug('external ftp connection used, skipping close')
            return

        try:
            logger.debug('Attempting to close ftp connection')
            self._ftp.close()
        except:
            logger.exception('Caught exception attempting to close connection')

    def delete_file(self, remote_file):
        """Deletes file specified by `remote_file`

           This method will delete file specified by
           `remote_file`.  If there is an error information can be
           obtained by calling `self.get_error_msg()`  It is assumed
           `connect()` has been called on this object.
           :param remote_file: full path to remote file to delete
           :returns: True upon success, false otherwise
        """
        self._error_msg = None
        start_time = int(time.time())
        try:
            if remote_file is None:
                self._error_msg = 'remote_file None'
                return False

            remote_path = os.path.normpath(remote_file)
            logger.debug('Deleting file : ' + remote_path)

            try:
                result = self._ftp.delete(remote_path)
                logger.debug(' Delete operation returned: ' + str(result))
            except Exception as e:
                logger.exception('Caught exception deleting file' +
                                 remote_path)
                self._error_msg = 'Unable to delete ' + remote_path +\
                                  ' : ' + str(e)
                return False

            return True
        finally:
            self._duration = int(time.time()) - start_time
            logger.debug('Delete operation took ' +
                         str(self._duration) + ' seconds')

    def download_file(self, remote_file, local_file):
        """Downloads file specified by `remote_file` to `local_file`

           This method will download file specified by
           `remote_file` to `local_file.  If there is an error information can
           be obtained by calling `self.get_error_msg()`  It is assumed
           `connect()` has been called on this object.
           :param remote_file: full path to remote file to download
           :param local_file: full path to download remote file to
           :returns: True upon success, false otherwise
        """
        self._error_msg = None
        start_time = int(time.time())
        try:
            if remote_file is None:
                self._error_msg = 'remote_file None'
                return False

            if local_file is None:
                self._error_msg = 'local_file None'
                return False

            remote_path = os.path.normpath(remote_file)
            local_path = os.path.normpath(local_file)
            logger.debug('Downloading file : ' + remote_path + ' to ' +
                         local_path)

            try:
                self._ftp.get(remote_path, local=local_path)
            except Exception as e:
                logger.exception('Caught exception downloading file' +
                                 remote_path + ' to ' + local_path)
                self._error_msg = ('Unable to download ' + remote_path +
                                   ' to ' + local_path + ' : ' + str(e))
                return False

            return True
        finally:
            self._duration = int(time.time()) - start_time
            logger.debug('Download operation took ' +
                         str(self._duration) + ' seconds')

    def list_dirs(self, remote_dir):
        """Gets list of all directories found in `remote_dir`

           This method gets a list of directories in `remote_dir`
           If there is an error information can be
           obtained by calling `self.get_error_msg()`  It is assumed
           `connect()` has been called on this object.
           :param remote_dir: full path to remote directory to examine
           :returns: None for failure and list upon success
        """
        self._error_msg = None
        start_time = int(time.time())
        try:
            if remote_dir is None:
                self._error_msg = 'remote_dir None'
                return None

            remote_path = os.path.normpath(remote_dir)
            logger.debug('Examining : ' + remote_path)

            dirlist = []
            try:
                for paths in self._ftp.list(remote_dir, extra=True):
                    if paths['directory'] == 'd':
                        if paths['name'] == '.':
                            continue
                        if paths['name'] == '..':
                            continue
                        dirlist.append(paths['name'])

            except Exception as e:
                logger.exception('Caught exception examining ' +
                                 remote_path)
                self._error_msg = ('Unable to get directory list for ' +
                                   remote_path +
                                   ' : ' + str(e))
                return None

            return dirlist
        finally:
            self._duration = int(time.time()) - start_time
            logger.debug('list directory operation took ' +
                         str(self._duration) + ' seconds')

    def list_files(self, remote_dir):
        """Gets list of all files found in `remote_dir`

           This method gets a list of files in `remote_dir`
           If there is an error information can be
           obtained by calling `self.get_error_msg()`  It is assumed
           `connect()` has been called on this object.
           :param remote_dir: full path to remote directory to examine
           :returns: None for failure and list upon success
        """
        self._error_msg = None
        start_time = int(time.time())
        try:
            if remote_dir is None:
                self._error_msg = 'remote_dir None'
                return None

            remote_path = os.path.normpath(remote_dir)
            logger.debug('Examining : ' + remote_path)

            filelist = []
            try:
                for paths in self._ftp.list(remote_dir, extra=True):
                    if paths['directory'] == '-':
                        filelist.append(paths['name'])

            except Exception as e:
                logger.exception('Caught exception examining ' +
                                 remote_path)
                self._error_msg = ('Unable to get file list for ' +
                                   remote_path +
                                   ' : ' + str(e))
                return None

            return filelist
        finally:
            self._duration = int(time.time()) - start_time
            logger.debug('list file operation took ' +
                         str(self._duration) + ' seconds')

    def upload_file_direct(self, file, remote_dir, remote_file_name):
        """Uploads file to remote server

           This method will upload the file to the `remote_dir` using the
           `remote_file_name` as the file name.  If there is an error
           information can be obtained by calling `self.get_error_msg()`
           :param file: full path to file to upload
           :param remote_dir: full path to remote directory to upload file to
           :param remote_file_name: name to use for file uploaded
           :returns: True upon success, false otherwise
        """
        self._error_msg = None
        self._bytes_transferred = 0
        self._files_transferred = 0
        start_time = int(time.time())
        try:
            if file is None:
                self._error_msg = 'File passed in is None'
                return False

            if not os.path.isfile(file):
                self._error_msg = file + ' is not a file'
                return False

            if remote_dir is None:
                self._error_msg = 'remote_dir is None'
                return False

            if remote_file_name is None:
                self._error_msg = 'remote_file_name is None'
                return False

            remote_path = os.path.normpath(remote_dir +
                                           os.sep + remote_file_name)
            logger.debug('Direct uploading file: ' + file + ' to ' +
                         remote_path)

            try:
                size = self._ftp.put(file, remote_path)

                logger.debug('  Uploaded ' + str(size) + ' bytes')
                self._bytes_transferred += size
                self._files_transferred += 1
            except Exception as e:
                logger.exception('Caught exception direct uploading file' +
                                 file + ' to ' + remote_path)
                self._error_msg = 'Unable to upload ' + file + ' to ' +\
                                  remote_path + ' : ' + str(e)
                return False
            return True
        finally:
            self._duration = int(time.time()) - start_time
            logger.debug('End of direct_file_upload operation took ' +
                         str(self._duration) + ' seconds')

    def _upload_file(self, file):
        """Uploads file to remote server

           This method assumes a valid ftp connection
           has been made with self._ftp

           :param: file full path to file to upload
        """
        if file is None:
            logger.error('File passed in is None')
            return

        if not os.path.isfile(file):
            logger.warning(file + ' is not a file')
            return

        remote_path = os.path.normpath(self.get_remote_dir() +
                                       os.sep + file)
        logger.debug('Uploading file: ' + file + ' to ' + remote_path)

        size = self._ftp.put(file, remote_path)

        logger.debug('  Uploaded ' + str(size) + ' bytes')
        self._bytes_transferred += size
        self._files_transferred += 1

    def upload_files(self, list_of_files):
        """Uploads files in `list_of_files` to ftp server

           Using ftp configuration set in constructor this
           method connects to the ftp server and uploads
           the files in `list_of_files` in a mirror fashion.
           For example, if `remote_dir` is set to:

           /foo

           and the files in `list_of_files` is:

           /home/bob/data.1
           /home/joe/data.2
           /var/blah.txt

           The files would be uploaded as follows:

           /home/bob/data.1 => /foo/home/bob/data.1
           /home/joe/data.2 => /foo/home/joe/data.2
           /var/blah.txt => /foo/var/blah.txt

           ftpretty is used for ftp upload and it makes
           sure the directories exist on remote server
           before uploading.

           This method will return False as soon as an
           ftp upload raises an exception.

           :returns: True upon success, False otherwise
        """
        self._error_msg = None
        self._bytes_transferred = 0
        self._files_transferred = 0
        start_time = int(time.time())
        try:
            if list_of_files is None:
                logger.warning('list of files is None')
                self._error_msg = 'List of files passed in was None'
                return True

            if len(list_of_files) is 0:
                logger.debug('No files to upload')
                self._error_msg = 'No files to upload'
                return True

            try:
                logger.debug('Uploading ' + str(len(list_of_files)) + ' files')
                for file in list_of_files:
                    self._upload_file(file)
            except:
                logger.exception('Caught exception')
                self._error_msg = 'Error during upload'
                return False
            return True
        finally:
            self._duration = int(time.time()) - start_time
            logger.debug('End of upload_files operation took ' +
                         str(self._duration) + ' seconds')


class WebDavFileTransfer(FileTransfer):
    """Implements FileUploader class by enabling upload of files to WebDav

    """
    def __init__(self, config):
        """Constructor
        """
        super(WebDavFileTransfer, self).__init__(config)

    def connect(self):
        if self._alt_ftp_con is None:
            try:

                logger.debug('Connecting to ' +
                             str(self.get_host()) + ' with user ' +
                             str(self.get_user()))
                self._ftp = easywebdav.connect(self.get_host(),
                                               username=self.get_user(),
                                               password=self.get_password(),
                                               protocol='https')
                return True
            except:
                logger.exception('Unable to connect to host')
                self._error_msg = 'Unable to connect to host'
                return False
        self._ftp = self._alt_ftp_con
        return True

    def download_file(self, remote_file, local_file):
        """Downloads file specified by `remote_file` to `local_file`

           This method will download file specified by
           `remote_file` to `local_file.  If there is an error information can
           be obtained by calling `self.get_error_msg()`  It is assumed
           `connect()` has been called on this object.
           :param remote_file: full path to remote file to download
           :param local_file: full path to download remote file to
           :returns: True upon success, false otherwise
        """
        self._error_msg = None
        start_time = int(time.time())
        try:
            if remote_file is None:
                self._error_msg = 'remote_file None'
                return False

            if local_file is None:
                self._error_msg = 'local_file None'
                return False

            remote_path = os.path.normpath(remote_file)
            local_path = os.path.normpath(local_file)
            logger.debug('Downloading file : ' + remote_path + ' to ' +
                         local_path)

            try:
                self._ftp.download(remote_path, local_path)
            except Exception as e:
                logger.exception('Caught exception downloading file' +
                                 remote_path + ' to ' + local_path)
                self._error_msg = ('Unable to download ' + remote_path +
                                   ' to ' + local_path + ' : ' + str(e))
                return False

            return True
        finally:
            self._duration = int(time.time()) - start_time
            logger.debug('Download operation took ' +
                         str(self._duration) + ' seconds')

    def upload_file_direct(self, file, remote_dir, remote_file_name):
        """Uploads file to remote server

           This method will upload the file to the `remote_dir` using the
           `remote_file_name` as the file name.  If there is an error
           information can be obtained by calling `self.get_error_msg()`
           :param file: full path to file to upload
           :param remote_dir: full path to remote directory to upload file to
           :param remote_file_name: name to use for file uploaded
           :returns: True upon success, false otherwise
        """
        self._error_msg = None
        self._bytes_transferred = 0
        self._files_transferred = 0
        start_time = int(time.time())
        try:
            if file is None:
                self._error_msg = 'File passed in is None'
                return False

            if not os.path.isfile(file):
                self._error_msg = file + ' is not a file'
                return False

            if remote_dir is None:
                self._error_msg = 'remote_dir is None'
                return False

            if remote_file_name is None:
                self._error_msg = 'remote_file_name is None'
                return False

            remote_path = os.path.normpath(remote_dir +
                                           os.sep + remote_file_name)
            logger.debug('Direct uploading file: ' + file + ' to ' +
                         remote_path)

            try:
                self._ftp.upload(file, remote_path)
                size = os.path.getsize(file)
                logger.debug('  Uploaded ' + str(size) + ' bytes')
                self._bytes_transferred += size
                self._files_transferred += 1
            except Exception as e:
                logger.exception('Caught exception direct uploading file' +
                                 file + ' to ' + remote_path)
                self._error_msg = 'Unable to upload ' + file + ' to ' +\
                                  remote_path + ' : ' + str(e)
                return False
            return True
        finally:
            self._duration = int(time.time()) - start_time
            logger.debug('End of direct_file_upload operation took ' +
                         str(self._duration) + ' seconds')



#!/usr/bin/env python

import commands
import os
import glob
import logging
import time
import re
logger = logging.getLogger()
logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.DEBUG )

#s2_result_path = "/data/celpp/2015/dataset.week.47/stage.2.blastnfilter"
#pdb_protein_location = "/data/pdb.extracted"
#full_copy_location = ""
#target_copy_location = ""

def extract_info_from_s2(stage_2_out):
    info_dic = {}
    f = open(stage_2_out,"r")
    lines = f.readlines()
    f.close()
    for line in lines:
        #print line.split(",")[0]
        info_title = line.split(",")[0]
        info_value_1 = line.split(",")[1] 
        if line.split(",")[0] == "inchi":
            info_title = line.split(", InChI=")[0] 
            info_value_1 = line.split(", InChI=")[1].split("\n")[0]
        else:
            try:
                info_value_2 = line.split(",")[2].split("\n")[0].strip()
                info_title = line.split(",")[0]
                info_value_1 = line.split(",")[1].strip() 
            except:
                info_value_2 = False
                info_title = line.split(",")[0]
                info_value_1 = line.split(",")[1].split("\n")[0].strip()
                pass
        #if not 
        if not info_value_2:
            #just use the first item if there are multiple entries (use the highest resolution one)
            if info_title not in info_dic:
                info_dic[info_title] = info_value_1
        else:
            if info_title not in info_dic:
                info_dic[info_title] = (info_value_1,info_value_2) 
    return info_dic

#copy all the txt files from the output stage 2 location and create folder for each of this named by the query entry
#check if it finished later needed 
    
def ligand_prepare(ligand_smile, out_lig_file):
    if os.path.isfile(out_lig_file):
        logging.info('Ligand file %s is already prepared. Skipping.' %(out_lig_file))
        return True
    commands.getoutput("$SCHRODINGER/ligprep -WAIT -i 0 -nt -ns -ismi %s -omae %s"%(ligand_smile, out_lig_file) )
    return os.path.isfile(out_lig_file)

def align_proteins (target_idtein, pre_prepare_protein, post_prepare_protein):
    commands.getoutput("$SCHRODINGER/utilities/structalign %s %s"%(target_idtein, pre_prepare_protein))
    rotated_protein = "rot-" + pre_prepare_protein
    if os.path.isfile(rotated_protein):
        commands.getoutput("mv %s %s"%(rotated_protein, post_prepare_protein))
        return True
    else:
        return False

def split_complex (part, complex_file, out_split):
    commands.getoutput("$SCHRODINGER/run split_structure.py -many_files -m %s %s %s"%(part, complex_file, out_split))
    out_receptor = os.path.splitext(out_split)[0] + "_receptor1"+ os.path.splitext(out_split)[1]
    if os.path.isfile(out_receptor):
        return out_receptor 
    else:
        return False

def prepare_protein (protein_file, prepared_protein, sleep_time = 300):
    #here the prepared protein should has ".maegz" as extension
    commands.getoutput("$SCHRODINGER/utilities/prepwizard %s %s"%(protein_file, prepared_protein))
    time.sleep(sleep_time) 
    if os.path.isfile(prepared_protein):
        return True
    else:
        return False


def main_proteinprep ( challenge_data_path, pdb_protein_path, working_folder ):
    os.chdir(working_folder)
    current_dir_layer_1 = os.getcwd()
#     all_stage_2_out = glob.glob("%s/*.txt"%s2_result_path)
#     summary_file = "%s/summary.txt"%s2_result_path
#     if summary_file in all_stage_2_out:
#         all_stage_2_out.remove(summary_file)
#     for single_stage_2_out in all_stage_2_out: 
#         target_id = os.path.basename(single_stage_2_out).split(".txt")[0]
#         valid_candidates[target_id] = []
#         if not os.path.isdir(target_id):
#             os.mkdir(target_id)
#         os.chdir(target_id)
#     #+++++++++++++++++++++
#     #get the infromation from stage 2 output
#         commands.getoutput("cp %s ." %(single_stage_2_out))
#         query_dic = extract_info_from_s2(target_id + ".txt")
# 
#     ######################
#     #step 1, check if there is a protein start with LMCSS if not then don't need to continue
#     ######################
#         if not "LMCSS" in query_dic:
#             logging.info("For this target protein: %s, there is no protein sharing the LMCSS ligand witt. Not able to generate Docking grid, pass for this case..."%target_id)
#             os.chdir(current_dir_layer_1)     
#             continue
#         elif len(query_dic["LMCSS"]) != 2:
#             logging.info("For this target protein: %s, the laregest protein has wrong number of informations associate with this id..."%target_id)
#             os.chdir(current_dir_layer_1)
#             continue
#         elif not "inchi" in query_dic:
#             logging.info("For this target protein: %s, there is no inchi info for the ligand..."%target_id)
#             os.chdir(current_dir_layer_1)
#             continue
#         else:
#             logging.info("============Start to work in this target protein: %s============"%target_id)
#             #raw_input()
#         ######################
#         #step 2, if there is a LMCSS protein then copy this portein to this folder and generate the center of the ligand
#         ######################
#             LMCSS_candidate_id = query_dic["LMCSS"][0]
#             LMCSS_candidate_filename = "LMCSS-%s_%s.pdb" %(target_id, LMCSS_candidate_id)
#             LMCSS_pdb_folder_name = LMCSS_candidate_id[1:3]
#             LMCSS_ent_file = "pdb" + LMCSS_candidate_id  + ".ent"
#             LMCSS_pdbloc = os.path.join(pdb_protein_path, LMCSS_pdb_folder_name, LMCSS_ent_file)
#             if not os.path.isfile(LMCSS_pdbloc):
#                 logging.info("Unable to find the ent file associate with the LMCSS candidate pdb: %s at location %s"%(LMCSS_candidate_id, LMCSS_pdbloc)) 
#                 os.chdir(current_dir_layer_1)
#                 continue
#             else:
#                 commands.getoutput("cp %s %s"%(LMCSS_pdbloc, LMCSS_candidate_filename))
#                 valid_candidates[target_id].append(LMCSS_candidate_filename)
# 
#         ######################
#         #step 3, get the center of the ligand in the LMCSS pdb
#         ######################
#         try:
#             LMCSS_ligand = query_dic["LMCSS"][1] 
#         except:
#             logging.info("There is no ligand information for this LMCSS candidate protein: %s"%(LMCSS_candidate_id))
#             os.chdir(current_dir_layer_1)
#             continue
#         ligand_center = get_center (LMCSS_candidate_filename, LMCSS_ligand) 
#         if not ligand_center:
#             logging.info("Unable to find the center of the ligand for the LMCSS candidate pdb: %s"%(LMCSS_candidate_id))
#             os.chdir(current_dir_layer_1)
#             continue
#         else:
#             center_file = open("center.txt", "w")
#             center_file.writelines(ligand_center)
#             center_file.close()
#         ######################
#         #step 4, prepare the ligand 
#         ######################
#         #a. generate the smile string from inchi 
#         ligand_inchi = "InChI=" + query_dic["inchi"]
#         inchi_f = open("tmp.inchi", "w")
#         inchi_f.writelines(ligand_inchi)
#         inchi_f.close()
#         inchi_filename = "tmp.inchi"
#         smile_filename = "tmp.smi"
#         #print "babel -iinchi %s  -osmi ---errorlevel 1"%inchi_filename
#         commands.getoutput("babel -iinchi %s  -osmi %s ---errorlevel 1"%(inchi_filename, smile_filename))
#         #raw_input()
#         if os.path.isfile(smile_filename):
#             smile_file = open(smile_filename, "r")
#             ligand_smile = smile_file.readlines()
#             smile_file.close()
#         else:
#             logging.info( "Convert from inchi:%s to SMILES failed"%(ligand_inchi))
#         #finish generating the smiles, then try to prepared the ligand 
#         ##############ligand prep step starts here##############
#         #if not ligand_prepare(ligand_smile, "ligand.mae"):
#         if not ligand_prepare(smile_filename, "ligand.mae"):
#             logging.info("Unable to prepare the ligand for this query protein:%s"%target_id)
#             os.chdir(current_dir_layer_1)
#             continue 
#         else:
#         ########################################################
#         #step 5, align all proteins
#             for method_type in ("SMCSS", "hiResHolo", "hiResApo"):
#                 if method_type in query_dic:
#                     #copy to here
#                     if len(query_dic[method_type]) == 2:
#                         #have both id and ligand info
#                         method_type_id = query_dic[method_type][0]
#                     else:
#                         #have only id info 
#                         method_type_id = query_dic[method_type]
#                     ## The pdb database is sorted by the middle two characters of the pdb code
#                     method_type_folder_name = method_type_id[1:3]
#                     method_ent_file = "pdb" + method_type_id  + ".ent"
#                     method_pdbloc = os.path.join(pdb_protein_path, method_type_folder_name, method_ent_file)
#                     if not os.path.isfile(method_pdbloc):
#                         logging.info("Unable to find the ent file associate with the %s pdb with ID: %s at location %s"%(method_type, method_type_id, LMCSS_pdbloc))
#                         os.chdir(current_dir_layer_1)
#                         continue
#                     else:
#                         method_type_filename = "%s-%s_%s.pdb" %(method_type, target_id, method_type_id)
#                         commands.getoutput("cp %s %s"%(method_pdbloc, method_type_filename))
#                         valid_candidates[target_id].append(method_type_filename)
#                     align_proteins (LMCSS_candidate_filename, method_type_filename, method_type_filename)
 
         
    ## Get all potential target directories and candidates within
    valid_candidates = {}
    #target_ligands = {}
    pot_target_dirs = list(os.walk(challenge_data_path))[0][1]
    #target_ids = []
    # Ensure that the directories are valid
    for pot_target_id in pot_target_dirs:
        os.chdir(current_dir_layer_1)
        # Does it look like a pdb id?
        if len(pot_target_id) != 4:
            logging.info('Filtering potential target directories: %s is not 4 characters long. Skipping' %(pot_target_id))
            continue
        commands.getoutput('mkdir %s' %(pot_target_id))
        #target_ids.append(pot_target_dir)
        valid_candidates[pot_target_id] = []
        target_dir_path = os.path.join(challenge_data_path, pot_target_id)

        # Pull in the ligand inchi
        lig_smiles_files = glob.glob('%s/lig_*.smi' %(target_dir_path))
        if len(lig_smiles_files) != 1:
            logging.info('Unable to find unambiguous ligand smiles for %s - glob returned %r' %(pot_target_id, lig_smiles_files))
            continue
        lig_smiles_file = lig_smiles_files[0]
        local_smiles_file = os.path.basename(lig_smiles_file)
        dest_smiles_file = os.path.join(pot_target_id, local_smiles_file)
        commands.getoutput('cp %s %s' %(lig_smiles_file, dest_smiles_file))

    
        center_file = os.path.join(target_dir_path,'center.txt')
        commands.getoutput('cp %s %s' %(center_file, pot_target_id))

        
        # Copy in each valid candidate
        for candidate_file in glob.glob('%s/*-%s_*.pdb' %(target_dir_path, pot_target_id)):
            # The LMCSS ligand will be in a pdb file called something like celpp_week19_2016/1fcz/LMCSS-1fcz_1fcz-156-lig.pdb
            # We want to make sure we don't treat this like a receptor
            if 'lig.pdb' in candidate_file:
                continue
            commands.getoutput('cp %s %s' %(candidate_file, pot_target_id))
            candidate_local_file = os.path.basename(candidate_file)
            valid_candidates[pot_target_id].append((local_smiles_file, candidate_local_file))
                
    for target_id in valid_candidates.keys():
        os.chdir(target_id)
        ######################
        #step 6, prepare all proteins
        ######################
        for smiles_filename, candidate_filename in valid_candidates[target_id]:

            
            
            ## Parse the candidate name 
            ## Get the method type, target, and candidate info from the filename
            # for example, this will parse 'hiResApo-5hib_2eb2_docked.mol' into [('hiResApo', '5hib', '2eb2')]
            
            parsed_name = re.findall('([a-zA-Z0-9]+)-([a-zA-Z0-9]+)_([a-zA-Z0-9]+)-?([a-zA-Z0-9]*).pdb', candidate_filename)
            if len(parsed_name) != 1:
                logging.info('Failed to parse docked structure name "%s". Parsing yielded %r' %(candidate_filename, parsed_name))
                continue
            candidate_structure_type = parsed_name[0][0]
            candidate_structure_target = parsed_name[0][1]
            candidate_structure_candidate = parsed_name[0][2]
            candidate_structure_ligand = parsed_name[0][2]
            
            # Prepare the ligand
            if not ligand_prepare(smiles_filename, smiles_filename.replace('.smi','_prepared.mae')):
                logging.info("Unable to prepare the ligand for this query protein:%s"%target_id)
                #os.chdir(current_dir_layer_1)
                continue 

            # Split the complex 
            #candidate_prefix = candidate_filename.replace('.pdb','')
            candidate_prefix = '%s-%s_%s' %(candidate_structure_type,
                                            candidate_structure_target,
                                            candidate_structure_candidate)
            
            out_split = candidate_prefix+ "_split.pdb"
            out_receptor = split_complex("pdb", candidate_filename, out_split)
            
            if not(out_receptor):
                logging.info("Unable to split this protein:%s"%(candidate_filename))
                continue
            
            logging.info("Successfully split this protein:%s, go to preparation step"%(candidate_filename))
            prepared_protein_maegz = candidate_prefix + "_prepared.mae"
            #pass the wizard sleep time here
            preparation_result = prepare_protein(out_receptor,prepared_protein_maegz, 180 )
            if not preparation_result:
                logging.info("Unable to prepare this protein:%s"%(out_split))
                continue                
            #convert into pdb format
            prepared_candidate_filename = candidate_prefix + "_prepared.pdb"
            commands.getoutput("$SCHRODINGER/utilities/pdbconvert -imae %s -opdb %s"%(prepared_protein_maegz, prepared_candidate_filename))
            logging.info("Successfully prepared this protein:%s"%(prepared_candidate_filename))
        os.chdir(current_dir_layer_1)
                    
                

if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-p", "--pdbdb", metavar = "PATH", help = "PDB DATABANK which we will dock into")
    parser.add_argument("-c", "--candidatedir", metavar="PATH", help = "PATH where we could find the stage 2 output")
    parser.add_argument("-o", "--outdir", metavar = "PATH", help = "PATH where we run stage 3")
    #parser.add_option("-s", "--sleep", metavar = "VALUE", help = "Sleep time for protein prep")
    #parser.add_option("-u", "--update", default = False, action = "store_true", help = "update the protein generation and docking step")
    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.DEBUG )
    opt = parser.parse_args()
    pdb_location = opt.pdbdb
    challenge_data_path = opt.candidatedir
    result_path = opt.outdir
    #sleep_time = opt.sleep
    #running under this dir
    running_dir = os.getcwd()
    main_proteinprep(challenge_data_path, pdb_location, result_path)
    #move the final log file to the result dir
    log_file_path = os.path.join(running_dir, 'final.log')
    commands.getoutput("mv %s %s"%(log_file_path,result_path))




__author__ = 'j5wagner'

import re

class ReadText(object):
    """
    CELPP <pdbid>.txt file reader
    """
    ## A dictionary of regular expressions for parsing target txt file lines
    PARSING_DICT = {'query':         '(\S+)',
                    'ph':            '(\S+)',
                    'ligand':        '([A-Za-z0-9]+)',
                    'inchi':         '(InChI=.*)',
                    'size':          '([0-9]+)',
                    'rotatable_bond':'([0-9]+)',
                    'LMCSS':         (['cand_id','lig_name','chain','size','mcss_size','resolution'],
                                      '([a-zA-Z0-9]{4}), ([a-zA-Z0-9]+), chain: (.), [(]size:[ ]([0-9]+), mcss_size:[ ]([0-9]+), resolution:[ ]*([0-9.]+)[)]\w*'),

                    'SMCSS':         (['cand_id','lig_name','chain','size','mcss_size','resolution'],
                                      '([a-zA-Z0-9]{4}), ([a-zA-Z0-9]+), chain: (.), [(]size:[ ]*([0-9]+), mcss_size:[ ]*([0-9]+), resolution:[ ]*([0-9.]+)[)]\w*'),

                    'hiResHolo':     (['cand_id','lig_name','chain','resolution'],
                                      '([a-zA-Z0-9]{4}), ([a-zA-Z0-9]+), chain: (.), [(]resolution:[ ]*([0-9.]+)[)]\w*'),

                    'hiTanimoto':    (['cand_id','lig_name','chain','tanimoto_similarity', 'resolution'],
                                      '([a-zA-Z0-9]{4}), ([a-zA-Z0-9]+), chain: (.), [(]tanimoto_similarity:[ ]*([0-9.]+), resolution:[ ]*([0-9.]+)[)]\w*'),

                    'hiResApo':      (['cand_id'],
                                      '([a-zA-Z0-9]{4})\w*'),
                    }
    
    ## A list of line types that return a dict instead of a single value
    DICT_RETURN_KEYS = ['LMCSS', 
                        'SMCSS',
                        'hiResHolo',
                        'hiTanimoto',
                        'hiResApo']

    def parse_line(self, line):
        """ 
        Parses a single line of a CELPP target .txt file
        :param: the line for processing
        :return: string - the key for the given line
        :return: string or dict - the value for the given line
        """
        line_split = line.split(',')
        txt_key = line_split[0]
        if txt_key[0] == '#':
            #print 'Skipping commmented line %s' %(line)
            return None, None

        if not(txt_key in self.PARSING_DICT.keys()):
            #raise Exception('Unable to parse line %r: key %s is invalid' %(line, key))
            #print 'Unable to parse line %r: key %s is invalid' %(line, txt_key)
            raise Exception('Unable to parse line %r: key %s is invalid' %(line, txt_key))
            return None, None

        value = ','.join(line_split[1:])
        #print
        #print 
        #print 'line', line
        if txt_key in self.DICT_RETURN_KEYS:
            data_keys = self.PARSING_DICT[txt_key][0]
            parsing_re = self.PARSING_DICT[txt_key][1]

            #print 'data_keys', data_keys
            #print 'parsing_re', parsing_re
            parsed_value = re.findall(parsing_re, value)
            #print 'parsed_value', parsed_value

            # Fix layering problem for candidates with only one field
            if not(type(parsed_value[0]) is tuple):
                parsed_value = tuple([parsed_value])
    

            if len(parsed_value[0]) != len(data_keys):
                #print "Didn't parse as many data values as data keys"
                raise Exception("Didn't parse as many data values as data keys")
                return None, None
            return_dict = {}
            for data_key, value in zip(data_keys,parsed_value[0]):
                #if data_key == 'other_info':
                    # Parse the value. 
                return_dict[data_key] = return_dict.get(data_key,[]) + [value]
            return txt_key, return_dict

        else:
            parsing_re = self.PARSING_DICT[txt_key]
            #print 'parsing_re', parsing_re
            parsed_value = re.findall(parsing_re, value)
            if len(parsed_value) != 1:
                #print "Parsing %s using RE %s returned %r. This parsing must return one unique match. Skipping line." %(value, parsing_re, parsed_value)
                raise Exception("Parsing %s using RE %s returned %r. This parsing must return one unique match. Skipping line." %(value, parsing_re, parsed_value))
                return None, None
            return txt_key, parsed_value[0]

    def parse_txt(self, txt_name):
        """Parses a CELPP target .txt file, returning a
        dictionary. Normal lines (eg "ph") are parsed into a
        key:[value] pair (eg. 'ph':['7.4']). As candidate information
        lines have a variable amount of data, they are parsed into a
        dictionary (eg. 'hiTanimoto':[{'cand_id': ['4di5'],
        'lig_name': ['DPO'], 'resolution': ['2.3'],
        'tanimoto_similarity': ['1.0'], 'chain': ['A']}] 
        :param: txt_name - the name of the txt file to be parsed 
        :return: a dictionary of all the information from the txt file.
        """

        data_dict = {}
        with open(txt_name) as fo:
            for line in fo:
                key, parsed_value = self.parse_line(line)
                if key is None:
                    continue
                    
                data_dict[key] = data_dict.get(key,[]) + [parsed_value]

        #print 
        #print
        #print
        #for key in data_dict.keys():
            #print key, ':', data_dict[key]
        return data_dict
                    




__author__ = 'churas'

import unittest
import tempfile
import os.path
import stat

"""
test_evaluation
--------------------------------

Tests for `evaluation` module.
"""
import shutil
import os

from mock import Mock

from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.dataimport import DataImportTask
from d3r.celpp.blastnfilter import BlastNFilterTask
from d3r.celpp.evaluation import EvaluationTaskFactory
from d3r.celpp.evaluation import EvaluationTask
from d3r.celpp.glide import GlideTask
from d3r.celpp.evaluation import PathNotDirectoryError
from d3r.celpp.participant import ParticipantDatabaseFromCSVFactory
from d3r.celpp.participant import ParticipantDatabase
from d3r.celpp.participant import Participant
from d3r.celpp.task import SmtpEmailer
from d3r.celpp.evaluation import EvaluationEmailer


class TestEvaluation(unittest.TestCase):
    def setUp(self):
        pass

    def test_get_uploadable_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            task = EvaluationTask(temp_dir, 'glide',
                                  GlideTask(temp_dir, params), params)
            # try with no dir
            self.assertEqual(task.get_uploadable_files(), [])

            # try with empty dir
            task.create_dir()
            self.assertEqual(task.get_uploadable_files(), [])

            # try with final log
            final_log = os.path.join(task.get_dir(),
                                     EvaluationTask.FINAL_LOG)
            open(final_log, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 1)
            flist.index(final_log)

            # try with RMSD.txt
            rmsd = os.path.join(task.get_dir(),
                                EvaluationTask.RMSD_TXT)
            open(rmsd, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            flist.index(rmsd)

            # try with empty pbdid dir
            pbdid = os.path.join(task.get_dir(), '8www')
            os.mkdir(pbdid)
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            flist.index(rmsd)

            # try with score/rot-LMCSS_doc_pv_complex1.pdb
            score = os.path.join(pbdid, 'score')
            os.mkdir(score)
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)

            LMCSS = os.path.join(score, 'LMCSS-1fcz_1fcz_docked_complex.pdb')
            open(LMCSS, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 3)
            flist.index(LMCSS)

            # try with score/rot-SMCSS_doc_pv_complex1.pdb
            SMCSS = os.path.join(score, 'SMCSS-1fcz_2lbd_docked_complex.pdb')
            open(SMCSS, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 4)
            flist.index(SMCSS)

            # try with score/rot-hiResApo_doc_pv_complex1.pdb
            hiResApo = os.path.join(score,
                                    'hiResHolo-1fcz_1fcy_docked_complex.pdb')
            open(hiResApo, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 5)
            flist.index(hiResApo)

            # try with score/rot-hiResHolo_doc_pv_complex1.pdb
            hiResHolo = os.path.join(score,
                                     'hiTanimoto-1fcz_1fcz_docked_complex.pdb')
            open(hiResHolo, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 6)
            flist.index(hiResHolo)

            # try with score/crystal.pdb
            crystal = os.path.join(score, 'crystal.pdb')
            open(crystal, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 7)
            flist.index(crystal)

            # try with RMSD.pickle
            rmsdpickle = os.path.join(task.get_dir(),
                                      EvaluationTask.RMSD_PICKLE)
            open(rmsdpickle, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 8)
            flist.index(rmsdpickle)

            # try with stderr/stdout files
            errfile = os.path.join(task.get_dir(), 'evaluate.py.stderr')
            open(errfile, 'a').close()
            outfile = os.path.join(task.get_dir(), 'evaluate.py.stdout')
            open(outfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 10)
            flist.index(crystal)
            flist.index(hiResHolo)
            flist.index(hiResApo)
            flist.index(SMCSS)
            flist.index(LMCSS)
            flist.index(errfile)
            flist.index(outfile)
            flist.index(final_log)
            flist.index(rmsd)
            flist.index(rmsdpickle)
        finally:
            shutil.rmtree(temp_dir)

    def test_evaluationtaskfactory_update_priorities_of_tasks(self):
        params = D3RParameters()
        params.hi = True
        etf = EvaluationTaskFactory('/foo', params)

        plist = [Participant('1name', '1d3rusername', '12345',
                             '1email@email.com',
                             priority=4),
                 Participant('4name', '4d3rusername', '5678_foo',
                             '4email@email.com',
                             priority=9)]

        pdb = ParticipantDatabase(plist)

        # one task not updated
        dtask = D3RTask('/foo', params)
        dtask.set_name('33' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        task = EvaluationTask('/foo', dtask.get_name(),
                              dtask, params)
        self.assertEqual(task.get_priority(), 0)
        etasks = etf._update_priorities_of_tasks([task], pdb)
        self.assertEqual(len(etasks), 1)
        self.assertEqual(etasks[0].get_priority(), 0)

        # two tasks both updated
        dtask = D3RTask('/foo', params)
        dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        taskone = EvaluationTask('/foo', dtask.get_name(),
                                 dtask, params)
        self.assertEqual(taskone.get_priority(), 0)

        dtask = D3RTask('/foo', params)
        dtask.set_name('5678_foo' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        tasktwo = EvaluationTask('/foo', dtask.get_name(),
                                 dtask, params)
        self.assertEqual(tasktwo.get_priority(), 0)

        etasks = etf._update_priorities_of_tasks([taskone, tasktwo], pdb)
        self.assertEqual(len(etasks), 2)
        self.assertEqual(etasks[0].get_priority(), 4)
        self.assertEqual(etasks[1].get_priority(), 9)

    def test_evaluationtaskfactory_sort_tasks_by_participant_priority(self):
        params = D3RParameters()
        params.hi = True
        etf = EvaluationTaskFactory('/foo', params)

        # test with no tasks
        self.assertEqual(etf._sort_tasks_by_participant_priority(None, None),
                         None)

        # test with NO participant database
        self.assertEqual(etf._sort_tasks_by_participant_priority([params],
                                                                 None),
                         [params])

        plist = [Participant('1name', '1d3rusername', '12345',
                             '1email@email.com',
                             priority=4),
                 Participant('4name', '4d3rusername', '5678_foo',
                             '4email@email.com',
                             priority=9)]

        pdb = ParticipantDatabase(plist)
        # two tasks both updated
        dtask = D3RTask('/foo', params)
        dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        taskone = EvaluationTask('/foo', dtask.get_name(),
                                 dtask, params)
        self.assertEqual(taskone.get_priority(), 0)

        dtask = D3RTask('/foo', params)
        dtask.set_name('5678_foo' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        tasktwo = EvaluationTask('/foo', dtask.get_name(),
                                 dtask, params)
        self.assertEqual(tasktwo.get_priority(), 0)

        etasks = etf._sort_tasks_by_participant_priority([taskone, tasktwo],
                                                         pdb)
        self.assertEqual(len(etasks), 2)
        self.assertEqual(etasks[0].get_priority(), 9)
        self.assertEqual(etasks[0].get_name(), '5678_foo' +
                         EvaluationTask.EXT_SUBMISSION_SUFFIX)

        self.assertEqual(etasks[1].get_priority(), 4)
        self.assertEqual(etasks[1].get_name(), '12345' +
                         EvaluationTask.EXT_SUBMISSION_SUFFIX)

    def test_evaluationtaskfactory_constructor(self):
        params = D3RParameters()
        params.hi = True
        stf = EvaluationTaskFactory('/foo', params)
        self.assertEquals(stf.get_args().hi, True)
        self.assertEquals(stf.get_path(), '/foo')

    def test_get_evaluation_tasks_invalid_path(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            path = os.path.join(temp_dir, 'doesnotexist')
            stf = EvaluationTaskFactory(path, params)
            try:
                stf.get_evaluation_tasks()
                self.fail('Expected PathNotDirectoryError')
            except PathNotDirectoryError:
                pass

        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_tasks_on_empty_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            stf = EvaluationTaskFactory(temp_dir, params)
            task_list = stf.get_evaluation_tasks()
            self.assertEquals(len(task_list), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_tasks_on_dir_with_lower_stages_dirs(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            dataimport = DataImportTask(temp_dir, params)
            blast = BlastNFilterTask(temp_dir, params)
            os.mkdir(os.path.join(temp_dir, dataimport.get_dir_name()))
            os.mkdir(os.path.join(temp_dir, blast.get_dir_name()))

            stf = EvaluationTaskFactory(temp_dir, params)
            task_list = stf.get_evaluation_tasks()
            self.assertEquals(len(task_list), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_tasks_on_dir_with_webdata_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            os.mkdir(os.path.join(temp_dir,
                                  EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                  EvaluationTaskFactory.WEB_DATA_SUFFIX))

            stf = EvaluationTaskFactory(temp_dir, params)
            task_list = stf.get_evaluation_tasks()
            self.assertEquals(len(task_list), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_tasks_on_with_valid_completed_algo_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            glidetask = GlideTask(temp_dir, params)
            glidetask.create_dir()

            open(os.path.join(glidetask.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            etask = EvaluationTask(temp_dir,
                                   glidetask.get_name() + '.' +
                                   EvaluationTaskFactory.SCORING_SUFFIX,
                                   glidetask,
                                   params)
            etask.create_dir()
            open(os.path.join(etask.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            stf = EvaluationTaskFactory(temp_dir, params)
            task_list = stf.get_evaluation_tasks()
            self.assertEquals(len(task_list), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_tasks_on_with_valid_algo_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            glidedir = os.path.join(temp_dir,
                                    EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                    'glide')
            os.mkdir(glidedir)
            open(os.path.join(glidedir, D3RTask.COMPLETE_FILE), 'a').close()
            stf = EvaluationTaskFactory(temp_dir, params)
            task_list = stf.get_evaluation_tasks()
            self.assertEquals(len(task_list), 1)
            self.assertEquals(task_list[0].get_name(), 'glide.evaluation')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_tasks_on_with_two_valid_algo_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            glidedir = os.path.join(temp_dir,
                                    EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                    'glide')
            os.mkdir(glidedir)
            open(os.path.join(glidedir, D3RTask.COMPLETE_FILE), 'a').close()

            freddir = os.path.join(temp_dir,
                                   EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                   'fred')
            os.mkdir(freddir)
            open(os.path.join(freddir, D3RTask.COMPLETE_FILE), 'a').close()

            stf = EvaluationTaskFactory(temp_dir, params)
            task_list = stf.get_evaluation_tasks()
            self.assertEquals(len(task_list), 2)
            self.assertNotEquals(task_list[0].get_name(),
                                 task_list[1].get_name())

        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_tasks_on_with_one_invalid_algo(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            glidedir = os.path.join(temp_dir,
                                    EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                    'glide')
            os.mkdir(glidedir)
            open(os.path.join(glidedir, D3RTask.ERROR_FILE), 'a').close()

            stf = EvaluationTaskFactory(temp_dir, params)
            task_list = stf.get_evaluation_tasks()
            self.assertEquals(len(task_list), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_tasks_on_with_one_valid_and_one_invalid_algo(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            glidedir = os.path.join(temp_dir,
                                    EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                    'glide')
            os.mkdir(glidedir)
            open(os.path.join(glidedir, D3RTask.ERROR_FILE), 'a').close()
            freddir = os.path.join(temp_dir,
                                   EvaluationTaskFactory.DOCKSTAGE_PREFIX +
                                   'fred')
            os.mkdir(freddir)
            open(os.path.join(freddir, D3RTask.COMPLETE_FILE), 'a').close()

            stf = EvaluationTaskFactory(temp_dir, params)
            task_list = stf.get_evaluation_tasks()
            self.assertEquals(len(task_list), 1)
            self.assertEquals(task_list[0].get_name(), 'fred.evaluation')
        finally:
            shutil.rmtree(temp_dir)

    def test_evaluationtask_constructor(self):
        params = D3RParameters()
        # no dock task found so it cannot run
        docktask = D3RTask('/blah', params)
        docktask.set_name('foo')
        docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)

        evaluation = EvaluationTask('/blah', 'foo.evaluation',
                                    docktask, params)
        self.assertEquals(evaluation.get_name(), 'foo.evaluation')
        self.assertEquals(evaluation.get_stage(), 7)
        self.assertEqual(evaluation.get_priority(), 0)
        evaluation.set_priority(4)
        self.assertEqual(evaluation.get_priority(), 4)

    def test_can_run(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            # no dock task found so it cannot run
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)

            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            self.assertEqual(evaluation.can_run(), False)
            self.assertEqual(evaluation.get_error(),
                             'foo task has notfound status')

            # docktask  running
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.START_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            self.assertEqual(evaluation.can_run(), False)
            self.assertEqual(evaluation.get_error(),
                             'foo task has start status')

            # docktask failed
            error_file = os.path.join(docktask.get_dir(),
                                      D3RTask.ERROR_FILE)
            open(error_file, 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            self.assertEqual(evaluation.can_run(), False)
            self.assertEqual(evaluation.get_error(),
                             'foo task has error status')

            # docktask success
            os.remove(error_file)
            open(os.path.join(docktask.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            self.assertEqual(evaluation.can_run(), True)
            self.assertEqual(evaluation.get_error(), None)

            # evaluation task exists already
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.create_dir()
            self.assertEqual(evaluation.can_run(), False)
            self.assertEqual(evaluation.get_error(),
                             evaluation.get_dir_name() +
                             ' already exists and status is unknown')

            # evaluation task already complete
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            open(os.path.join(evaluation.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            self.assertEqual(evaluation.can_run(), False)
            self.assertEqual(evaluation.get_error(), None)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_can_run_is_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            # return immediately cause can_run is false
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.run()
            self.assertEqual(evaluation.get_error(),
                             'foo task has notfound status')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_evaluation_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.run()
            self.assertEqual(evaluation.get_error(),
                             'evaluation not set')
            # test files get created
            self.assertEqual(os.path.isdir(evaluation.get_dir()),
                             True)
            errfile = os.path.join(evaluation.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_pdbdb_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.evaluation = 'true'
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.run()
            self.assertEqual(evaluation.get_error(),
                             'pdbdb not set')
            # test files get created
            self.assertEqual(os.path.isdir(evaluation.get_dir()),
                             True)
            errfile = os.path.join(evaluation.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_evaluation_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.evaluation = 'false'
            params.pdbdb = '/data/pdb'
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.run()
            self.assertEqual(evaluation.get_error(), None)
            # test file gets created
            compfile = os.path.join(evaluation.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)

            stderr = os.path.join(evaluation.get_dir(),
                                  'false.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(evaluation.get_dir(),
                                  'false.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_evaluation_is_not_found(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.evaluation = '/bin/doesnotexist'
            params.pdbdb = '/data/pdb'
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.run()
            self.assertEqual(evaluation.get_error(),
                             'Caught Exception trying to run ' +
                             '/bin/doesnotexist --pdbdb /data/pdb ' +
                             '--dockdir ' +
                             docktask.get_dir() + ' --outdir ' +
                             evaluation.get_dir() +
                             ' : [Errno 2] No such file or directory')

            # test files get created
            errfile = os.path.join(evaluation.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_succeeds_no_emailer(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.evaluation = 'true'
            params.pdbdb = '/data/pdb'
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.run()
            self.assertEqual(evaluation.get_error(), None)
            # test files get created
            errfile = os.path.join(evaluation.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), False)

            compfile = os.path.join(evaluation.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)
            stderr = os.path.join(evaluation.get_dir(),
                                  'true.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(evaluation.get_dir(),
                                  'true.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_succeeds_no_emailer_withtimeout(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.evaluation = 'true'
            params.evaluationtimeout = 100
            params.evaluationtimeoutkilldelay = 10
            params.pdbdb = '/data/pdb'
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.run()
            self.assertEqual(evaluation.get_error(), None)
            # test files get created
            errfile = os.path.join(evaluation.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), False)

            compfile = os.path.join(evaluation.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)
            stderr = os.path.join(evaluation.get_dir(),
                                  'true.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(evaluation.get_dir(),
                                  'true.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_due_to_timeout_no_emailer(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            foo_script = os.path.join(temp_dir, 'foo.py')
            params.evaluation = foo_script
            params.evaluationtimeout = 1
            params.evaluationtimeoutkilldelay = 2
            params.pdbdb = '/data/pdb'

            # create fake blastnfilter script that makes csv files
            f = open(foo_script, 'w')
            f.write('#! /usr/bin/env python\n\n')
            f.write('import time\n')
            f.write('time.sleep(360)\n')
            f.flush()
            f.close()
            os.chmod(foo_script, stat.S_IRWXU)

            docktask = D3RTask(temp_dir, params)
            docktask.set_name('foo')
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, 'foo.evaluation',
                                        docktask, params)
            evaluation.run()
            self.assertEqual(evaluation.get_error(), None)

            # test files get created
            compfile = os.path.join(evaluation.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)
            stderr = os.path.join(evaluation.get_dir(),
                                  'foo.py.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(evaluation.get_dir(),
                                  'foo.py.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_succeeds_with_emailer(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.evaluation = 'true'
            params.pdbdb = '/data/pdb'
            docktask = D3RTask(temp_dir, params)
            docktask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
            docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
            docktask.create_dir()
            open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            evaluation = EvaluationTask(temp_dir, docktask.get_name(),
                                        docktask, params)
            plist = [Participant('1name', '1d3rusername', '12345',
                                 'bob@bob.com,joe@joe.com')]
            smtpemailer = SmtpEmailer()
            mockserver = D3RParameters()
            mockserver.sendmail = Mock()
            mockserver.quit = Mock()
            smtpemailer.set_alternate_smtp_server(mockserver)
            emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
            emailer.set_alternate_smtp_emailer(smtpemailer)
            evaluation.set_evaluation_emailer(emailer)
            evaluation.run()
            self.assertEqual(evaluation.get_error(), None)
            # test files get created
            errfile = os.path.join(evaluation.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), False)

            compfile = os.path.join(evaluation.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)
            stderr = os.path.join(evaluation.get_dir(),
                                  'true.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(evaluation.get_dir(),
                                  'true.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
            res = evaluation.get_email_log().endswith('\nSent evaluation '
                                                      'email to: bob@bob.com,'
                                                      ' joe@joe.com\n')
            self.assertTrue(res)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluation_summary(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = EvaluationTask(temp_dir, 'foo', None, params)
            # test where no RMSD.txt file is found
            val = task.get_evaluation_summary()
            self.assertEqual(val,
                             '\nEvaluation of docking\n'
                             '=====================\nNo ' +
                             task.get_rmsd_txt() + ' file found.\n')

            # test with valid RMSD.txt file
            task.create_dir()
            f = open(task.get_rmsd_txt(), 'w')
            f.write('LMCSS\n1fcz 0.465\n')
            f.flush()
            f.close()
            val = task.get_evaluation_summary()
            self.assertEqual(val,
                             '\nEvaluation of docking\n'
                             '=====================\n'
                             'LMCSS\n1fcz 0.465\n\n')

            # test where reading RMSD.txt throws exception
            os.chmod(task.get_rmsd_txt(), 0)
            val = task.get_evaluation_summary()
            self.assertTrue(val.startswith('\nEvaluation of docking\n'
                                           '=====================\n'
                                           'Unable to generate evaluation'
                                           ' summary ('))
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_external_submission_email_body(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            dtask = D3RTask('/foo', params)
            dtask.set_name('foo')
            task = EvaluationTask(temp_dir, dtask.get_name(), dtask, params)
            task.create_dir()
            emailer = EvaluationEmailer(None, None)
            task.set_evaluation_emailer(emailer)
            rmsd = task.get_rmsd_txt()
            f = open(rmsd, 'w')
            f.write('  LMCSS  XXX\n1fcz  0.2  0.4\n')
            f.flush()
            f.close()
            subject, body = emailer\
                ._generate_external_submission_email_body(task)
            self.assertEqual(subject,
                             '[d3rcelpp] Week 0 evaluation results for foo')
            self.assertEqual(body, 'Dear CELPP Participant,\n\nHere are your '
                                   'docking evaluation results '
                                   '(RMSD, Angstroms) '
                                   'for CELPP week '
                                   '0\n\n\nEvaluation of docking'
                                   '\n=====================\n  LMCSS  XXX\n'
                                   '1fcz  0.2  0.4\n\n\n\nSincerely,\n\n'
                                   'CELPP Automation')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_rmsd_txt(self):
        params = D3RParameters()
        task = EvaluationTask('/ha', 'foo', None, params)
        self.assertEqual(task.get_rmsd_txt(),
                         os.path.join('/ha', task.get_dir_name(),
                                      EvaluationTask.RMSD_TXT))

    def test_get_rmsd_pickle(self):
        params = D3RParameters()
        task = EvaluationTask('/ha', 'foo', None, params)
        self.assertEqual(task.get_rmsd_pickle(),
                         os.path.join('/ha', task.get_dir_name(),
                                      EvaluationTask.RMSD_PICKLE))

    def test_is_external_submission(self):
        params = D3RParameters()
        dtask = D3RTask('/ha', params)
        task = EvaluationTask('/ha', 'foo', dtask, params)
        dtask.set_name('blah')
        self.assertEqual(task.is_external_submission(), False)
        dtask.set_name('blah' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        self.assertEqual(task.is_external_submission(), True)

    def test_get_participant_database_no_csv_file_found(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            etf = EvaluationTaskFactory(temp_dir, params)
            pdb = etf._get_participant_database()
            self.assertEqual(pdb, None)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_participant_database_with_valid_csvfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            dimport = DataImportTask(temp_dir, params)
            dimport.create_dir()
            csvfile = dimport.get_participant_list_csv()
            f = open(csvfile, 'w')
            f.write('name,d3rusername,guid,email\n')
            f.write('joe,jj,123,j@j.com\n')
            f.flush()
            f.close()
            etf = EvaluationTaskFactory(temp_dir, params)
            pdb = etf._get_participant_database()
            p = pdb.get_participant_by_guid('123')
            self.assertEqual(p.get_email(), 'j@j.com')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_guid_for_task_dock_is_none(self):
        params = D3RParameters()
        task = EvaluationTask('/foo', 'blah' +
                              EvaluationTask.EXT_SUBMISSION_SUFFIX,
                              None,
                              params)
        self.assertEqual(task.get_guid_for_task(),
                         None)

    def test_get_guid_for_task(self):
        params = D3RParameters()
        dtask = D3RTask('/foo', params)
        dtask.set_name('123' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        task = EvaluationTask('/foo', dtask.get_name(),
                              dtask,
                              params)
        self.assertEqual(task.get_guid_for_task(),
                         '123')

    def test_eval_emailer_append_to_message_log(self):
        emailer = EvaluationEmailer(None, None)
        self.assertEqual(emailer.get_message_log(), None)
        emailer._append_to_message_log('hi\n')
        self.assertEqual(emailer.get_message_log(), 'hi\n')
        emailer._append_to_message_log('how\n')
        self.assertEqual(emailer.get_message_log(), 'hi\nhow\n')

    def test_eval_emailer_get_external_submitter_email_none_from_guid(self):
        emailer = EvaluationEmailer(None, None)
        params = D3RParameters()
        task = EvaluationTask('/foo', None, None, params)
        self.assertEqual(emailer._get_external_submitter_email(task), None)
        self.assertEqual(emailer.get_message_log(),
                         '\nUnable to extract guid\n')

    def test_get_external_submitter_email_participant_not_found(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            dimport = DataImportTask(temp_dir, params)
            dimport.create_dir()
            csvfile = dimport.get_participant_list_csv()
            f = open(csvfile, 'w')
            f.write('name,d3rusername,guid,email\n')
            f.write('joe,jj,123,j@j.com\n')
            f.flush()
            f.close()
            fac = ParticipantDatabaseFromCSVFactory(csvfile)
            params = D3RParameters()
            dtask = D3RTask('/foo', params)
            dtask.set_name('444' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
            task = EvaluationTask(temp_dir,
                                  dtask.get_name(),
                                  dtask, params)
            emailer = EvaluationEmailer(fac.get_participant_database(), None)
            self.assertEqual(emailer._get_external_submitter_email(task),
                             None)
            self.assertEqual(emailer.get_message_log(),
                             '\nNo participant found with guid: 444\n')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_external_submitter_email_no_participant_email(self):

        params = D3RParameters()
        dtask = D3RTask('/foo', params)
        dtask.set_name('444' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        task = EvaluationTask('/foo',
                              dtask.get_name(),
                              dtask, params)
        plist = [Participant('1name', '1d3rusername', '444',
                             None)]
        emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
        self.assertEqual(emailer._get_external_submitter_email(task), None)
        self.assertEqual(emailer.get_message_log(),
                         '\nEmail not set for participant\n')

    def test_get_external_submitter_email_valid(self):

        params = D3RParameters()
        dtask = D3RTask('/foo', params)
        dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        task = EvaluationTask('/foo',
                              dtask.get_name(),
                              dtask, params)
        plist = [Participant('1name', '1d3rusername', '12345',
                             'bob@bob.com')]
        # try single email address
        emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
        emails = emailer._get_external_submitter_email(task)
        self.assertEqual(emails[0], 'bob@bob.com')
        self.assertEqual(len(emails), 1)
        self.assertEqual(emailer.get_message_log(), None)

        # try multiple email address
        plist = [Participant('1name', '1d3rusername', '12345',
                             'bob@bob.com,joe@joe.com')]
        # try single email address
        emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
        emails = emailer._get_external_submitter_email(task)
        self.assertEqual(emails[0], 'bob@bob.com')
        self.assertEqual(emails[1], 'joe@joe.com')
        self.assertEqual(len(emails), 2)
        self.assertEqual(emailer.get_message_log(), None)

    def test_send_evaluation_email_none_task(self):
        emailer = EvaluationEmailer(None, None)
        emailer.send_evaluation_email(None)
        self.assertEqual(emailer.get_message_log(),
                         '\nTask passed in is None\n')

    def test_send_evaluation_email_clears_its_logs(self):
        emailer = EvaluationEmailer(None, None)
        emailer.send_evaluation_email(None)
        emailer.send_evaluation_email(None)

        self.assertEqual(emailer.get_message_log(),
                         '\nTask passed in is None\n')

    def test_send_evaluation_email_not_external_task(self):
        emailer = EvaluationEmailer(None, None)
        task = EvaluationTask('/foo', 'blah' +
                              EvaluationTask.EXT_SUBMISSION_SUFFIX,
                              None,
                              D3RParameters())
        emailer.send_evaluation_email(task)
        self.assertEqual(emailer.get_message_log(),
                         '\nNot an external submission\n')

    def test_send_evaluation_email_no_database(self):
        params = D3RParameters()
        dtask = D3RTask('/foo', params)
        dtask.set_name('444' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        task = EvaluationTask('/foo',
                              dtask.get_name(),
                              dtask, params)
        emailer = EvaluationEmailer(None, None)
        emailer.send_evaluation_email(task)
        self.assertEqual(emailer.get_message_log(),
                         '\nParticipant database is None cannot send docking '
                         'evaluation email!!!\n')

    def test_send_external_submission_email_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.program = 'foo'
            params.version = '1'
            dtask = D3RTask('/foo', params)
            dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
            task = EvaluationTask(temp_dir,
                                  dtask.get_name(),
                                  dtask, params)
            task.create_dir()
            f = open(task.get_rmsd_txt(), 'w')
            f.write('hi\n')
            f.flush()
            f.close()
            plist = [Participant('1name', '1d3rusername', '12345',
                                 'bob@bob.com')]
            # try single email address
            smtpemailer = SmtpEmailer()
            mockserver = D3RParameters()
            mockserver.sendmail = Mock()
            mockserver.quit = Mock()
            smtpemailer.set_alternate_smtp_server(mockserver)
            emailer = EvaluationEmailer(ParticipantDatabase(plist), None)

            emailer.set_alternate_smtp_emailer(smtpemailer)
            emailer.send_evaluation_email(task)
            mockserver.quit.assert_any_call()
            self.assertEqual(emailer.get_message_log(),
                             '\nSent evaluation email to: bob@bob.com\n')
            self.assertEqual(mockserver.sendmail.call_count, 1)

        finally:
            shutil.rmtree(temp_dir)

    def test_send_external_submission_email_sendmail_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.program = 'foo'
            params.version = '1'
            dtask = D3RTask('/foo', params)
            dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
            task = EvaluationTask(temp_dir,
                                  dtask.get_name(),
                                  dtask, params)
            plist = [Participant('1name', '1d3rusername', '12345',
                                 'bob@bob.com')]
            # try single email address
            smtpemailer = SmtpEmailer()
            mockserver = D3RParameters()
            mockserver.sendmail = Mock(side_effect=IOError('ha'))
            mockserver.quit = Mock()
            smtpemailer.set_alternate_smtp_server(mockserver)
            emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
            emailer.set_alternate_smtp_emailer(smtpemailer)
            emailer.send_evaluation_email(task)
            mockserver.quit.assert_any_call()
            self.assertEqual(emailer.get_message_log(),
                             '\nCaught exception trying to email '
                             'participant : Caught exception ha\n')

        finally:
            shutil.rmtree(temp_dir)

    def test_send_external_submission_email_no_submitter_email(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.program = 'foo'
            params.version = '1'
            dtask = D3RTask('/foo', params)
            dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
            task = EvaluationTask(temp_dir,
                                  dtask.get_name(),
                                  dtask, params)
            plist = [Participant('1name', '1d3rusername', '1234',
                                 'bob@bob.com')]
            # try single email address
            emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
            smtpemailer = SmtpEmailer()
            emailer.set_alternate_smtp_emailer(smtpemailer)
            emailer.send_evaluation_email(task)
            self.assertEqual(emailer.get_message_log(),
                             '\nNo participant found with guid: 12345\n')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_reply_to_address(self):
        # test get reply_to where no replytoaddress is set
        emailer = EvaluationEmailer(None, None)
        val = emailer._get_reply_to_address('bob@bob.com')
        self.assertEqual(val, 'bob@bob.com')

        emailer = EvaluationEmailer(None, 'joe@joe.com')
        val = emailer._get_reply_to_address('bob@bob.com')
        self.assertEqual(val, 'joe@joe.com')

    def test_get_smtp_emailer_valid(self):
        try:
            emailer = EvaluationEmailer(None, None)
            ss = emailer._get_smtp_emailer()
            self.assertTrue(ss is not None)
        finally:
            pass

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()
